// SongDump.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#define FILE_NUM 10
#define FILE_SIZE 1073709056
FILE* fps[FILE_NUM];
char fileNames[FILE_NUM][32] = 
{
	"DTSMUS00.DKD",
	"DTSMUS01.DKD",
	"DTSMUS02.DKD",
	"DTSMUS03.DKD",
	"DTSMUS04.DKD",
	"DTSMUS05.DKD",
	"DTSMUS06.DKD",
	"DTSMUS07.DKD",
	"DTSMUS08.DKD",
	"DTSMUS09.DKD"
};

void correctEndian(unsigned int* data, unsigned int size)
{
	unsigned char* p = (unsigned char*)data;

	for (unsigned int i = 0; i < size; i++)
	{
		data[i] = p[0] << 24 | p[1] << 16 | p[2] << 8 | p[3];
		p += 4;
	}
}

FILE* getFp(unsigned int offset)
{
	int fileIndex = offset / FILE_SIZE;
	FILE* fp = fps[fileIndex];

	if (fp == 0)
		fps[offset / FILE_SIZE] = fp = fopen(fileNames[fileIndex], "rb");

	if (fp == 0)
		printf("Unable to open file %s\n", fileNames[fileIndex]);

	return fp;
}

void closeFiles()
{
	for (int i = 0; i < FILE_NUM; i++)
	{
		if (fps[i] != 0)
			fclose(fps[i]);
	}
}

void dumpSong (char* song)
{
	FILE* fp;
	unsigned int tables[256];
	unsigned int table[256];
	unsigned int tableNumber;
	unsigned int songNumber = atoi(song);
	char songFileName[256];

	memset(fps, 0, sizeof(fps));

	// Read the tables
	fp = getFp(0x200);
	fseek(fp, 0x200, SEEK_SET);
	fread(tables, sizeof(tables), 1, fp);
	correctEndian(tables, sizeof(tables)/sizeof(int));
	// Adjust offsets
	for (int i = 0; i < sizeof(tables)/sizeof(int); i++)
	{
		tables[i] *= 0x800;
	}

	// Calculate the table of the song
	tableNumber = (int)ceil(songNumber / 256.0) - 1;

	// Read the table
	fp = getFp(tables[tableNumber]);
	fseek(fp, tables[tableNumber] % FILE_SIZE, SEEK_SET);
	fread(table, sizeof(table), 1, fp);
	correctEndian(table, sizeof(table)/sizeof(int));
	// Adjust offsets
	for (i = 0; i < sizeof(tables)/sizeof(int); i++)
	{
		table[i] += (tables[tableNumber] % FILE_SIZE);
	}

	// Calculate the song offset
	unsigned int songOffset = table[(songNumber - songNumber / 256 * 256 - 1)];
	unsigned int songLength = table[((songNumber + 1) - (songNumber + 1) / 256 * 256 - 1)] - songOffset;
	if (songOffset >= FILE_SIZE)
	{
		fclose(fp);
		printf("Song offset too large 0x%08X\n", songOffset);
		return;
	}

	printf("Song %05d, Offset:0x%08X, Length:0x%08X\n", songNumber, songOffset, songLength);
	sprintf(songFileName, "%05d.dat", songNumber);
	FILE* fpOut = fopen(songFileName, "wb");
	char* buffer = new char[songLength];
	fseek(fp, songOffset, SEEK_SET);
	fread(buffer, songLength, 1, fp);
	fwrite(buffer, songLength, 1, fpOut);
	fclose(fpOut);

	fclose(fp);
}

void findPattern2(FILE* fp, int startOffset, int seperation)
{
	int byte;
	unsigned char byteC = 0;
	unsigned char byteCount = 1;
	unsigned int offsetC = startOffset;
	unsigned int offset = 1;

	fseek(fp, startOffset, SEEK_SET);

	byteC = fgetc(fp);

	while ((byte = fgetc(fp)) != EOF)
	{
		if (offset % seperation == 0)
		{
			if (byteC == byte)
			{
				byteCount++;
			}
			else
			{
				if (byteCount > 5)
					printf("Offset: 0x%X, 0x%02X, %d\n", offsetC, byteC, byteCount);

				byteCount = 1;
				byteC = byte;
				offsetC = offset + startOffset;
			}
		}

		offset++;
	}

}

void findPattern(char* fileName, int seperation)
{
	FILE* fp = fopen(fileName, "rb");

	for (int i = 0; i < seperation; i++)
	{
		findPattern2(fp, i, seperation);
	}

	fclose(fp);
}

int main(int argc, char* argv[])
{
	dumpSong(argv[1]);
/*	
	for (int i = 1; i <= 27; i++)
	{
		printf("Seperation %d:\n", i);
		findPattern(argv[1], i);
	}
*/
	return 0;
}
